/***************************************************************************
                          kxine.cpp  -  description
                             -------------------
    begin                : Fri Jun 28 20:23:37 BST 2002
    copyright            : (C) 2002 by Mark Thomas
    email                : markbt@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxine.h"

#include <kapplication.h>
#include <kaboutapplication.h>

#include <qlayout.h>
#include <qcursor.h>

#include <iostream>

#define SPLITTER_WIDTH 12

using namespace std;

Kxine::Kxine(QWidget *parent, const char *name) : QWidget(parent, name)
{
  aColn = new KActionCollection (this, "KxineActionColn", application);

  QVBoxLayout *vbl = new QVBoxLayout (this);
  QSplitter *sp = new QSplitter (QSplitter::Horizontal, this);

  kxine = this;

  menu = new KxineMenu (this, "kxMenu");
  control = new KxineControl (this, "kxControl");
  sidebar = new KxineSideBar (sp, "kxSideBar");
  video = new KxineVideo (sidebar, menu, control, sp);
  videoControl = NULL;

  sp -> moveToFirst (video);
  sp -> setResizeMode (sidebar, QSplitter::KeepSize);

  vbl -> addWidget (menu);
  vbl -> addWidget (sp, 1);
  vbl -> addWidget (control);

  connect (menu, SIGNAL (appClose()), SLOT (close()));
  connect (menu, SIGNAL (aboutApplication()), SLOT (aboutApplication()));
  connect (menu, SIGNAL (toggleFullScreen(bool)), SLOT (toggleFullScreen(bool)));
  connect (menu, SIGNAL (toggleDeinterlace(bool)), SLOT (toggleDeinterlace(bool)));
  connect (menu, SIGNAL (frameAdjustSize ()), SLOT (frameAdjustSize ()));
  connect (menu, SIGNAL (openVideoControlWindow ()), SLOT (openVideoControlWindow()));

  connect (this, SIGNAL (moved()), video, SLOT (refresh()));

  setCaption ("Kxine Media Player");
  setMouseTracking (true);
  sp -> setMouseTracking (true);
  fullScreen = false;
  deinterlace = false;
  mouseTimerId = -1;
  frameAdjustSize ();
}

Kxine::~Kxine()
{
}

void
Kxine::clearQueue ()
{
  sidebar -> clearQueue (); 
}

void
Kxine::playFile (QString filename)
{
  sidebar -> setQueueMode ();
  video -> xineOpen (filename);
}

void
Kxine::enqueueFile (QString filename)
{
  sidebar -> enqueueMRL (filename);
}

void
Kxine::aboutApplication ()
{
  (new KAboutApplication (this, "about", false))->show ();
}

void
Kxine::toggleFullScreen (bool fs)
{
  if (fullScreen == fs)
    return;
  if (fullScreen)
    {
      // cancel fullscreen
      showNormal ();
      menu -> show ();
      control -> show ();
      sidebar -> show ();
      fullScreen = false;
      cerr << "resizing to " << oldSize.width() << " x " << oldSize.height() << endl;
      resize (oldSize);
      if (mouseTimerId != -1)
	{
	  killTimer (mouseTimerId);
	  mouseTimerId = -1;
	}
      else
	{
	  video -> setCursor (ArrowCursor);
	}
    }
  else
    {
      // go fullscreen
      oldSize = size ();
      cerr << "oldSize is " << oldSize.width() << " x " << oldSize.height() << endl;
      showFullScreen ();
      menu -> hide ();
      control -> hide ();
      sidebar -> hide ();
      fullScreen = true;
      video -> refresh();
      mouseTimerId = startTimer (1000);
    }
  menu -> setFullscreenCheck (fullScreen);
}

void
Kxine::toggleFrameVisible ()
{
  if (!fullScreen)
    {
       if (menu->isVisible())
         {
           menu -> hide ();
           sidebar -> hide ();
           control -> hide ();
         }
       else
         {
           menu -> show ();
           sidebar -> show ();
           control -> show ();
         }
    }
}

void
Kxine::toggleDeinterlace (bool deint)
{
  video -> xineSetProperty (VO_PROP_INTERLACED, deint ? 1 : 0);
}

void
Kxine::openVideoControlWindow ()
{
  if (videoControl != NULL)
    {
      videoControl -> setActiveWindow ();
      return;
    }

  KxineVideoControl *kvc = new KxineVideoControl (this);

  bool en;
  int min = 0, max = 0;

  en = video -> xineCheckCapability (VO_CAP_HUE);
  if (en)
    video -> xineGetPropertyMinMax (VO_PROP_HUE, &min, &max);
  kvc -> enableHue (en, min, max);

  en = video -> xineCheckCapability (VO_CAP_SATURATION);
  if (en)
    video -> xineGetPropertyMinMax (VO_PROP_SATURATION, &min, &max);
  kvc -> enableSaturation (en, min, max);

  en = video -> xineCheckCapability (VO_CAP_CONTRAST);
  if (en)
    video -> xineGetPropertyMinMax (VO_PROP_CONTRAST, &min, &max);
  kvc -> enableContrast (en, min, max);

  en = video -> xineCheckCapability (VO_CAP_BRIGHTNESS);
  if (en)
    video -> xineGetPropertyMinMax (VO_PROP_BRIGHTNESS, &min, &max);
  kvc -> enableBrightness (en, min, max);

  kvc -> setHue (video -> xineGetProperty (VO_PROP_HUE));
  kvc -> setContrast (video -> xineGetProperty (VO_PROP_CONTRAST));
  kvc -> setSaturation (video -> xineGetProperty (VO_PROP_SATURATION));
  kvc -> setBrightness (video -> xineGetProperty (VO_PROP_BRIGHTNESS));
 
  connect (kvc, SIGNAL (propertyChanged(int,int)),
           video, SLOT (xineSetProperty(int,int)));

  connect (kvc, SIGNAL (windowClosed()), SLOT (videoControlWindowClosed()));

  kvc -> show ();

  videoControl = kvc;
 
}

void
Kxine::videoControlWindowClosed ()
{
  videoControl = NULL;
}

void
Kxine::keyPressEvent (QKeyEvent * kev)
{
  switch (kev->key ())
    {
    case Key_Space:
      video -> xinePlayPause();
      break;
    case Key_F:
      toggleFullScreen (!fullScreen);
      break;
    case Key_Q:
      close ();
      break;
    case Key_F9:
      toggleFrameVisible ();
    }
}

void
Kxine::moveEvent (QMoveEvent *)
{
  emit moved ();
}

void
Kxine::mouseMoveEvent (QMouseEvent *mev)
{
  if (fullScreen)
    {
      if (control->isVisible ())
	{
	  if (mev->y () < control->y ()
	      && mev->y () > menu -> height ()
              && mev->x () < sidebar -> x () - SPLITTER_WIDTH)
	    {
	      control -> hide ();
	      menu ->hide ();
              sidebar -> hide ();
	      video -> refresh ();
	    }
	}
      else
	{
	  if (mev->y () >= height () - control -> height ()
	      || mev->y () <= menu -> height ()
              || mev->x () >= width () - sidebar -> width () - SPLITTER_WIDTH)
	    {
	      control -> show ();
	      menu -> show ();
              sidebar -> show ();
	      video -> refresh ();
	    }
	}

      if (mouseTimerId != -1)
	killTimer (mouseTimerId);
      else
	video -> setCursor (ArrowCursor);

      mouseTimerId = startTimer (1000);
    }
}


void
Kxine::timerEvent (QTimerEvent *tev)
{
  if (tev->timerId () == mouseTimerId)
    {
      killTimer (mouseTimerId);
      mouseTimerId = -1;
      video -> setCursor (BlankCursor);
    }
}

void
Kxine::closeEvent (QCloseEvent *cev)
{
  video -> xineExit ();
  delete video; video = NULL;
  delete control; control = NULL;
  delete menu; menu = NULL;
  cev -> accept ();
  KApplication::exit (0);
}

void
Kxine::frameAdjustSize ()
{
  if (!fullScreen)
    {
      // a bit of a hack...  FIXME
      QSize videoSize = video -> size ();
      QSize frameSize = size ();
      QSize videoSizeHint = video -> getVideoSize ();
      QSize newFrameSize =
        QSize (videoSizeHint.width() + frameSize.width() - videoSize.width(),
               videoSizeHint.height() + frameSize.height() - videoSize.height());
      resize (newFrameSize);
    }
}
